extern crate lru_cache;
extern crate hyper;
extern crate select;
extern crate retry;

pub mod pirates;
pub mod ninjas;
mod parser;
//mod retry;

#[macro_use]
extern crate log;

#[derive(Debug)]
pub enum Error {
    Http(hyper::error::Error),
    Io(std::io::Error),
    Parse(String),
    Unknown(String)
}

impl From<hyper::error::Error> for Error {
    fn from (err: hyper::error::Error) -> Error {
        Error::Http(err)
    }
}

impl From<std::io::Error> for Error {
    fn from (err: std::io::Error) -> Error {
        Error::Io(err)
    }
}

impl From<std::num::ParseIntError> for Error {
    fn from (err: std::num::ParseIntError) -> Error {
        Error::Parse(format!("{}", err))
    }
}

impl From<retry::Error<Error>> for Error {
    fn from (err: retry::Error<Error>) -> Error {
        match err {
            retry::Error::Operation { error, total_delay, tries } => {
                error
            }
        }
    }
}

pub trait Fighter {
    fn power (&self) -> u8;
}
